#!/usr/bin/env python3
import os
import subprocess
import gi.repository
gi.require_version('Budgie', '1.0')
from gi.repository import Budgie, GObject, Gtk


"""
Kangaroo
Author: Jacob Vlijm
Copyright © 2017-2019 Ubuntu Budgie Developers
Website=https://ubuntubudgie.org
This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or any later version. This
program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details. You
should have received a copy of the GNU General Public License along with this
program.  If not, see <https://www.gnu.org/licenses/>.
-"""


userhome = os.environ["HOME"]


# config path
dr = os.path.join(userhome, ".config", "budgie-extras", "folderjumper")
# settings files
showinvisible = os.path.join(dr, "showinvisible")
hidetooltips = os.path.join(dr, "hidetooltips")
defdirfile = os.path.join(dr, "defaultdir")


# tooltips
tooltip1 = "Left-click to open file, right-click to open its directory"
tooltip2 = "Right-click to open"


try:
    os.makedirs(dr)
except FileExistsError:
    pass


def get_defaultdir():
    try:
        return open(defdirfile).read().strip()
    except FileNotFoundError:
        return userhome


class Kangaroo(GObject.GObject, Budgie.Plugin):

    __gtype_name__ = "BudgieKangaroo"

    def __int__(self):
        GObject.Object.__init__(self)

    def do_get_panel_widget(self, uuid):
        return KangarooApplet(uuid)


class KangarooSettings(Gtk.Grid):

    def __init__(self, setting):

        super().__init__()

        self.setting = setting
        # grid & layout
        self.set_row_spacing(12)
        self.hidetooltips = Gtk.CheckButton("Hide tooltips")
        self.showinvisibles = Gtk.CheckButton("Show hidden files")
        element_hsizer1 = self.h_spacer(13)
        self.set_root = Gtk.Button("Set root directory")
        self.set_root.connect("clicked", self.get_directory)
        self.dir_entry = Gtk.Entry(editable=False)
        self.dir_entry.set_text(get_defaultdir())
        self.hidetooltips.set_active(os.path.exists(hidetooltips))
        self.showinvisibles.set_active(os.path.exists(showinvisible))
        self.hidetooltips.connect(
            "toggled", self.edit_settings, hidetooltips
        )
        self.showinvisibles.connect(
            "toggled", self.edit_settings, showinvisible
        )
        self.attach(self.hidetooltips, 1, 1, 1, 1)
        self.attach(self.showinvisibles, 1, 2, 1, 1)
        self.attach(element_hsizer1, 1, 3, 1, 1)
        self.attach(self.set_root, 1, 4, 1, 1)
        self.attach(self.dir_entry, 1, 5, 1, 1)
        self.show_all()

    def h_spacer(self, addwidth):
        # horizontal spacer
        spacegrid = Gtk.Grid()
        if addwidth:
            label1 = Gtk.Label()
            label2 = Gtk.Label()
            spacegrid.attach(label1, 0, 0, 1, 1)
            spacegrid.attach(label2, 0, 1, 1, 1)
            spacegrid.set_row_spacing(addwidth)
        return spacegrid

    def get_directory(self, button):
        try:
            directory = subprocess.check_output([
                "zenity", "--file-selection", "--directory",
            ]).decode("utf-8").strip()
        except subprocess.CalledProcessError:
            pass
        else:
            self.dir_entry.set_text(directory)
            open(defdirfile, "wt").write(directory)

    def edit_settings(self, button, settingsfile):
        state = button.get_active()
        if state:
            open(settingsfile, "wt").write("")
        else:
            try:
                os.remove(settingsfile)
            except FileNotFoundError:
                pass


class KangarooApplet(Budgie.Applet):

    manager = None

    def __init__(self, uuid):

        Budgie.Applet.__init__(self)
        self.uuid = uuid
        self.menu = Gtk.Menu()

        """
        unfortunately, if a menuitem has a submenu, it won't act on righ / left
        button press, connected to the menuitem. we therefore need to set the
        action globally for the menu as a whole. the downside is that (in this
        case)right- click still works on the last selected item if used outside
        the menu. no solution found. more a cosmetic issue than a use issue
        though. normally, most users will never notice. nevertheless, still
        looking for a solution.
        """

        self.menu.connect("button-press-event", self.open_onthree)
        box = Gtk.EventBox()
        self.add(box)
        icon = Gtk.Image.new_from_icon_name(
            "budgie-foldertrack-symbolic", Gtk.IconSize.MENU
        )
        box.add(icon)
        box.show_all()
        box.connect("button-press-event", self.popup_menu)
        self.fileselect = False
        self.currselectedfile = None
        self.show_all()
        self.menu.show_all()

    def popup_menu(self, *args):
        # refresh menu content on popup
        self.default_dir = get_defaultdir()
        for i in self.menu.get_children():
            self.menu.remove(i)
        self.hide_tooltips = os.path.exists(hidetooltips)
        self.show_invisibles = os.path.exists(showinvisible)
        self.create_level(self.default_dir, self.menu)
        self.menu.append(Gtk.SeparatorMenuItem())
        showhide_state = self.set_showhide()
        self.toggle_showhide = Gtk.MenuItem(showhide_state)
        self.toggle_showhide.connect("activate", self.toggle_visible)
        self.menu.append(self.toggle_showhide)
        self.show_all()
        self.menu.show_all()
        self.menu.popup(
            None, None, None, None, 0, Gtk.get_current_event_time()
        )

    def set_showhide(self):
        if os.path.exists(showinvisible):
            return "✓\tInvisible items"
        else:
            return "✕\tInvisble items"

    def toggle_visible(self, menu):
        state = os.path.exists(showinvisible)
        if not state:
            open(showinvisible, "wt").write("")
        else:
            try:
                os.remove(showinvisible)
            except FileNotFoundError:
                pass

    def check_invisible(self, f):
        # filter out invisoble files if set
        name = f.name
        return name, not any([name.startswith("."), name.endswith("~")])

    def tooltip(self, item, text):
        # set tooltip
        item.set_tooltip_text(text)

    def create_level(self, dr, master=None, *args):
        # create new menu layer items
        newitems = [[f, self.check_invisible(f)] for f in os.scandir(dr)]
        if not self.show_invisibles:
            newitems = [f for f in newitems if f[1][1]]
        newitems.sort(key=lambda x: x[1][0])
        # show "Empty" on empty folders. should show a > nevertheless
        if not newitems:
            firstsub = Gtk.MenuItem("Empty")
            master.append(firstsub)
        # if not empty, show content on select / activate
        else:
            for item in newitems:
                item_name = item[1][0]
                item = item[0]
                # item, firstsub
                firstsub = Gtk.MenuItem(item_name)
                master.append(firstsub)
                newpath = item.path
                if item.is_dir():
                    try:
                        secondsubitems = [
                            it.name for it in os.scandir(newpath)
                        ]
                        self.dressup_dirmenuitem(firstsub, item_name, newpath)
                    except PermissionError:
                        firstsub.set_label("✕\t" + item_name)
                else:
                    self.dressup_filemenuitem(firstsub, item_name, newpath)

    def dressup_filemenuitem(self, menuitem, itemname, newpath):
        # setup the file- menuitem
        menuitem.set_label("•\t" + itemname)
        menuitem.connect(
            "button-press-event", self.open_file_onone, newpath
        )
        if not self.hide_tooltips:
            self.tooltip(menuitem, tooltip1)
        menuitem.connect(
            "select", self.get_selectedfile, True, newpath
        )

    def dressup_dirmenuitem(self, menuitem, itemname, newpath):
        # setup the directory- menuitem
        secondsub = Gtk.Menu()
        menuitem.set_submenu(secondsub)
        menuitem.connect("activate", self.add_layer, newpath)
        menuitem.connect(
            "select", self.get_selectedfile, False, newpath
        )
        menuitem.set_label("⏍\t" + itemname)
        if not self.hide_tooltips:
            self.tooltip(menuitem, tooltip2)

    def add_layer(self, menuitem, newpath):
        # initiate a new, deeper layer
        newmenu = Gtk.Menu()
        menuitem.set_submenu(newmenu)
        self.create_level(newpath, newmenu)
        self.curr_subject = newpath
        menuitem.show_all()

    def get_selectedfile(self, widget, val, newpath):
        # get the currently selected file if applicable
        # sets False if latest selected is a dir
        self.fileselect = val
        self.currselectedfile = newpath

    def open_file_onone(self, menuitem, button, path):
        # open the selected file on left click
        if button.button == 1:
            self.curr_subject = path
            subprocess.Popen(["xdg-open", self.curr_subject])

    def open_onthree(self, event, button):
        # opens either the directory, if selected,
        # or the superior directory of a file, if selected
        if button.button == 3:
            if not self.fileselect:
                path = self.curr_subject
            else:
                path = self.currselectedfile[
                    :self.currselectedfile.rfind("/")
                ]
            subprocess.Popen(["xdg-open", path])
            self.menu.popdown()

    def do_get_settings_ui(self):
        """Return the applet settings with given uuid"""
        return KangarooSettings(self.get_applet_settings(self.uuid))

    def do_supports_settings(self):
        """Return True if support setting through Budgie Setting,
        False otherwise.
        """
        return True



#!/usr/bin/env python3
import os
import subprocess
import time
import gi
gi.require_version("Pango", "1.0")
gi.require_version('Gdk', '3.0')
from gi.repository import Pango, Gdk


"""
Budgie WindowMover
Author: Jacob Vlijm
Copyright © 2017-2019 Ubuntu Budgie Developers
Website=https://ubuntubudgie.org
This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or any later version. This
program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details. You
should have received a copy of the GNU General Public License along with this
program.  If not, see <http://www.gnu.org/licenses/>.
"""

settings_path = os.path.join(
    os.environ["HOME"], ".config", "budgie-extras", "wmover"
)

user = os.environ["USER"]
fpath = "/tmp/" + user + "_wmover_busy"
appletpath = os.path.dirname(os.path.abspath(__file__))
wmover_ismuted = os.path.join(settings_path, "muted")

try:
    os.makedirs(settings_path)
except FileExistsError:
    pass

# wm_classes to be ignored
ignore = [
    '"budgie-panel", "Budgie-panel"',
    '"desktop_window", "Nautilus"',
    '"plank", "Plank"',
    None,
]


def get(cmd):
    try:
        return subprocess.check_output(cmd).decode("utf-8")
    except subprocess.CalledProcessError:
        pass


def show_wmclass(wid):
    # get WM_CLASS from window- id
    try:
        return get(["xprop", "-id", wid, "WM_CLASS"]).split("=")[-1].strip()
    except (IndexError, AttributeError):
        pass


def get_wsdata():
    wsdata = get(["wmctrl", "-d"]).splitlines()
    return (len(wsdata), wsdata.index([l for l in wsdata if "*" in l][0]))


def run(cmd):
    try:
        subprocess.Popen(cmd)
    except TypeError:
        pass


def check_ypos(yres):
    # get active window, check y- position
    try:
        subj = get(["xdotool", "getactivewindow"])
        ydata = get(["xdotool", "getwindowgeometry", subj])
    except (subprocess.CalledProcessError, TypeError):
        return False, None
    else:
        ypos = int([l for l in ydata.splitlines()
                    if "Position" in l][0].split(",")[1].split()[0])
        return ypos > yres - 150, subj.strip()


def getres():
    # get the resolution from wmctrl
    resdata = get(["wmctrl", "-d"])
    res = [int(n) for n in resdata.split()[3].split("x")] if resdata else None
    return res


def area(x_area, y_area, xres, yres, x, y):
    # see if the mouse is in the hotspot
    center = xres / 2
    halfwidth = x_area / 2
    x_match = center - halfwidth < x < center + halfwidth
    y_match = y > yres - y_area
    return all([x_match, y_match])


def mousepos():
    return Gdk.get_default_root_window().get_pointer()[1:3]


def find_bar():
    return get(["xdotool", "search", "--class", "wmover"])


def callwindow(target, xres, yres):
    wtype = show_wmclass(target)
    if wtype in ignore:
        run(["notify-send", "-i", "wmover-panel",
             "WindowMover", "Please first activate a window."])
    else:
        runwindow(target, xres, yres)


def runwindow(target, xres, yres):
    # run the mover bar
    subprocess.Popen([
        os.path.join(appletpath, "moverbar"),
        target,
        str(xres),
        str(yres),
    ])
    time.sleep(0.5)
    limit_exist()


def get_font():
    key = ["org.gnome.desktop.wm.preferences", "titlebar-font"]
    fontdata = get(["gsettings", "get", key[0], key[1]]).strip("'")
    fdscr = Pango.FontDescription(fontdata)
    return Pango.FontDescription.get_family(fdscr)


def limit_exist():
    # make sure the bar stays on top *and* active, allow 5 seconds
    t = 1
    while True:
        time.sleep(1)
        if t >= 5:
            run(["wmctrl", "-ic", find_bar()])
            break
        else:
            try:
                if "w_moversplash" in get(["wmctrl", "-l"]):
                    run(["wmctrl", "-a", "w_moversplash"])
                else:
                    break
            except TypeError:
                pass

        if os.path.exists(fpath):
            t = 1
        else:
            t = t + 1



import gi.repository

gi.require_version('Budgie', '1.0')
from gi.repository import Budgie, GObject, Gtk, Gio
import os

"""
RotationLock
Author: David Mohammed
Copyright © 2017-2019 Ubuntu Budgie Developers
Website=https://ubuntubudgie.org
This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or any later version. This
program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details. You
should have received a copy of the GNU General Public License along with this
program.  If not, see <https://www.gnu.org/licenses/>.

screenlock icons
Icons made by Google from https://www.flaticon.com is licensed by
http://creativecommons.org/licenses/by/3.0/
"""


class BudgieRotationLock(GObject.GObject, Budgie.Plugin):
    """ This is simply an entry point into your Budgie Applet implementation.
        Note you must always override Object, and implement Plugin.
    """

    # Good manners, make sure we have unique name in GObject type system
    __gtype_name__ = "BudgieRotationLock"

    def __init__(self):
        """ Initialisation is important.
        """
        GObject.Object.__init__(self)

    def do_get_panel_widget(self, uuid):
        """ This is where the real fun happens. Return a new Budgie.Applet
            instance with the given UUID. The UUID is determined by the
            BudgiePanelManager, and is used for lifetime tracking.
        """
        return BudgieRotationLockApplet(uuid)


class BudgieRotationLockApplet(Budgie.Applet):
    """ Budgie.Applet is in fact a Gtk.Bin """
    manager = None

    def __init__(self, uuid):
        Budgie.Applet.__init__(self)
        self.box = Gtk.EventBox()
        self.lockicon = Gtk.Image.new_from_icon_name(
            "budgie-rotation-lock-button-symbolic",
            Gtk.IconSize.MENU,
        )

        self.unlockicon = Gtk.Image.new_from_icon_name(
            "budgie-rotation-button-symbolic",
            Gtk.IconSize.MENU,
        )

        if Gtk.get_major_version() == 3 and \
                Gtk.get_minor_version() == 18:
            # GTK+3.18
            schema = "org.gnome.settings-daemon.plugins.orientation"
            self.key = "active"
        else:
            # > GTK+3.18
            schema = "org.gnome.settings-daemon.peripherals.touchscreen"
            self.key = "orientation-lock"

        self.settings = Gio.Settings.new(schema)

        if self.settings.get_boolean(self.key):
            if self.key == "active":
                self.displayicon = self.unlockicon
            else:
                self.displayicon = self.lockicon
        else:
            if self.key == "active":
                self.displayicon = self.lockicon
            else:
                self.displayicon = self.unlockicon

        self.box.add(self.displayicon)
        self.add(self.box)
        self.box.show_all()
        self.show_all()
        self.box.connect("button-press-event", self.on_press)

    def on_press(self, box, arg):
        self.box.remove(self.displayicon)

        if self.settings.get_boolean(self.key):
            if self.key == "active":
                self.displayicon = self.lockicon
            else:
                self.displayicon = self.unlockicon
            self.settings.set_boolean(self.key, False)
        else:
            if self.key == "active":
                self.displayicon = self.unlockicon
            else:
                self.displayicon = self.lockicon

            self.settings.set_boolean(self.key, True)

        self.box.add(self.displayicon)
        self.box.show_all()



#!/usr/bin/python3

# This file is part of App Launcher

# Copyright © 2018-2019 Ubuntu Budgie Developers

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.


class App():

    def __init__(self, id, name, isActive):
        self.id = id
        self.name = name
        self.isActive = isActive
        self.info = None
        self.index = None

    def setId(self, id):
        self.id = id

    def getId(self):
        return self.id

    def getName(self):
        return self.name

    def setActive(self, isActive):
        self.isActive = isActive

    def getActive(self):
        return self.isActive

    def setInfo(self, info):
        self.info = info

    def getInfo(self):
        return self.info

    def setIndex(self, index):
        self.index = index

    def getIndex(self):
        return self.index



#!/usr/bin/python3

# This file is part of App Launcher

# Copyright © 2018-2019 Ubuntu Budgie Developers

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.


class FilterHelper():

    def __init__(self):
        self.TAG = "FilterHelper"

    def filteredAppsByName(self, apps, showOnPanel, searchText):
        filteredApps = []
        counter = 0
        for app in apps:
            if counter >= showOnPanel and \
                    searchText.lower() in app.getName().lower():
                filteredApps.append(app)
            counter += 1
        if searchText is "":
            filteredApps = apps
        return filteredApps



#!/usr/bin/python3

# This file is part of App Launcher

# Copyright © 2018-2019 Ubuntu Budgie Developers

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.


import json
from Log import Log
from Error import Error


class JsonHelper():

    def __init__(self):
        self.TAG = "JsonHelper"
        self.log = Log("budgie-app-launcher")

    def setDictIfNone(self, data, key):
        if data is not None and key is not None:
            if data.get(key, None) is None:
                data[key] = {}
            return data[key]

    def setListIfNone(self, data, key):
        if data is not None and key is not None:
            if data.get(key, None) is None:
                data[key] = []
            return data[key]

    def readData(self, filePath):
        try:
            with open(filePath, 'r+') as json_file:
                data = json.load(json_file)
                return data
        except Exception as e:
            self.log.e(self.TAG, Error.ERROR_1010, e)

    def writeData(self, filePath, data):
        try:
            with open(filePath, 'w+') as outfile:
                json.dump(data, outfile, indent=4)
        except Exception as e:
            self.log.e(self.TAG, Error.ERROR_1011, e)



import gi.repository

gi.require_version('Budgie', '1.0')
from gi.repository import Budgie, GObject, Gtk, Gio
import os
import clocktools as clt

"""
Budgie ShowTime
Author: Jacob Vlijm
Copyright © 2017-2019 Ubuntu Budgie Developers
Website=https://ubuntubudgie.org
This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or any later version. This
program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details. You
should have received a copy of the GNU General Public License along with this
program.  If not, see <http://www.gnu.org/licenses/>.
"""

css_data = """
.colorbutton {
  border-color: transparent;
  background-color: hexcolor;
  padding: 0px;
  border-width: 1px;
  border-radius: 4px;
}
.colorbutton:hover {
  border-color: hexcolor;
  background-color: hexcolor;
  padding: 0px;
  border-width: 1px;
  border-radius: 4px;
}
"""

colorpicker = os.path.join(clt.app_path, "colorpicker")
cpos_file = clt.pos_file


class BudgieShowTime(GObject.GObject, Budgie.Plugin):
    """ This is simply an entry point into your Budgie Applet implementation.
        Note you must always override Object, and implement Plugin.
    """

    # Good manners, make sure we have unique name in GObject type system
    __gtype_name__ = "BudgieShowTime"

    def __init__(self):
        """ Initialisation is important.
        """
        GObject.Object.__init__(self)

    def do_get_panel_widget(self, uuid):
        """ This is where the real fun happens. Return a new Budgie.Applet
            instance with the given UUID. The UUID is determined by the
            BudgiePanelManager, and is used for lifetime tracking.
        """
        return BudgieShowTimeApplet(uuid)


class BudgieShowTimeSettings(Gtk.Grid):
    def __init__(self, setting):

        super().__init__()
        self.setting = setting

        # files & colors
        self.tcolorfile = clt.timecolor
        self.dcolorfile = clt.datecolor
        self.mute_time = clt.mute_time
        self.mute_date = clt.mute_date
        self.twelvehrs = clt.twelve_hrs
        # grid & layout
        self.set_row_spacing(12)
        element_hsizer1 = self.h_spacer(13)
        self.attach(element_hsizer1, 0, 0, 1, 7)
        element_hsizer2 = self.h_spacer(25)
        self.attach(element_hsizer2, 2, 0, 1, 7)
        # time section
        self.runtime = Gtk.CheckButton("Show time")
        self.attach(self.runtime, 1, 1, 1, 1)
        self.twelve_hrs = Gtk.CheckButton("Twelve hours display")
        self.attach(self.twelve_hrs, 1, 2, 1, 1)
        # time color
        self.bholder1 = Gtk.Box()
        self.attach(self.bholder1, 1, 3, 1, 1)
        self.t_color = Gtk.Button()
        self.t_color.connect("clicked", self.pick_color, self.tcolorfile)
        self.t_color.set_size_request(10, 10)
        self.bholder1.pack_start(self.t_color, False, False, 0)
        timelabel = Gtk.Label(" Set time color")
        self.bholder1.pack_start(timelabel, False, False, 0)
        spacer1 = Gtk.Label("")
        self.attach(spacer1, 1, 4, 1, 1)
        # date section
        self.rundate = Gtk.CheckButton("Show date")
        self.attach(self.rundate, 1, 5, 1, 1)
        # date color
        self.bholder2 = Gtk.Box()
        self.attach(self.bholder2, 1, 6, 1, 1)
        self.d_color = Gtk.Button()
        self.d_color.connect("clicked", self.pick_color, self.dcolorfile)
        self.d_color.set_size_request(10, 10)
        self.bholder2.pack_start(self.d_color, False, False, 0)
        datelabel = Gtk.Label(" Set date color")
        self.bholder2.pack_start(datelabel, False, False, 0)
        spacer2 = Gtk.Label("")
        self.attach(spacer2, 1, 7, 1, 1)
        # position section
        # checkbox custom position
        self.setposbutton = Gtk.CheckButton("Set custom position (px)")
        self.attach(self.setposbutton, 1, 8, 1, 1)
        # position
        posholder = Gtk.Box()
        self.xpos = Gtk.Entry()
        self.xpos.set_width_chars(4)
        self.xpos_label = Gtk.Label("x: ")
        self.ypos = Gtk.Entry()
        self.ypos.set_width_chars(4)
        self.ypos_label = Gtk.Label(" y: ")
        self.apply = Gtk.Button("OK")
        for item in [
            self.xpos_label, self.xpos, self.ypos_label, self.ypos,
        ]:
            posholder.pack_start(item, False, False, 0)
        posholder.pack_end(self.apply, False, False, 0)
        self.attach(posholder, 1, 9, 1, 1)
        self.twelve_hrs.set_active(os.path.exists(self.twelvehrs))
        # set initial states
        timeshows = not os.path.exists(self.mute_time)
        dateshows = not os.path.exists(self.mute_date)
        self.runtime.set_active(timeshows)
        self.rundate.set_active(dateshows)
        self.set_timestate(timeshows)
        self.set_datestate(dateshows)
        self.runtime.connect("toggled", self.toggle_show, self.mute_time)
        self.rundate.connect("toggled", self.toggle_show, self.mute_date)
        self.twelve_hrs.connect("toggled", self.toggle_12, self.twelvehrs)
        # color buttons & labels
        distlabel = Gtk.Label("")
        self.attach(distlabel, 1, 10, 1, 1)
        noicon = Gtk.Label("Applet runs without a panel icon")
        self.attach(noicon, 1, 11, 1, 1)
        self.set_initialstate()
        self.setposbutton.connect("toggled", self.toggle_cuspos)
        self.apply.connect("clicked", self.get_xy)
        self.update_color()
        self.show_all()

    def set_timestate(self, val=None):
        val = val if val else not os.path.exists(self.mute_time)
        for item in [
            self.twelve_hrs, self.bholder1, self.t_color,
        ]:
            item.set_sensitive(val)

    def set_datestate(self, val=None):
        # could be included in set_timestate but, well...
        val = val if val else not os.path.exists(self.mute_date)
        for item in [self.bholder2, self.d_color]:
            item.set_sensitive(val)

    def set_initialstate(self):
        # set initial state of items in the custom position section
        state_data = clt.get_textposition()
        state = state_data[0]
        if state:
            self.xpos.set_text(str(state_data[1]))
            self.ypos.set_text(str(state_data[2]))
        for entr in [
            self.ypos, self.xpos, self.apply, self.xpos_label, self.ypos_label
        ]:
            entr.set_sensitive(state)
        self.setposbutton.set_active(state)

    def get_xy(self, button):
        x = self.xpos.get_text()
        y = self.ypos.get_text()
        # check for correct input
        try:
            newpos = [str(int(p)) for p in [x, y]]
            open(cpos_file, "wt").write("\n".join(newpos))
        except (FileNotFoundError, ValueError, IndexError):
            pass
        clt.restart_clock()

    def toggle_cuspos(self, button):
        newstate = button.get_active()
        for widget in [
            self.ypos, self.xpos, self.xpos_label, self.ypos_label, self.apply
        ]:
            widget.set_sensitive(newstate)
        if newstate is False:
            self.xpos.set_text("")
            self.ypos.set_text("")
            try:
                os.remove(cpos_file)
            except FileNotFoundError:
                pass
            else:
                clt.restart_clock()

    def h_spacer(self, addwidth):
        # horizontal spacer
        spacegrid = Gtk.Grid()
        if addwidth:
            label1 = Gtk.Label()
            label2 = Gtk.Label()
            spacegrid.attach(label1, 0, 0, 1, 1)
            spacegrid.attach(label2, 1, 0, 1, 1)
            spacegrid.set_column_spacing(addwidth)
        return spacegrid

    def toggle_12(self, button, file):
        # cannot be included in toggle_show, flipped boolean
        if not button.get_active():
            try:
                os.remove(file)
            except FileNotFoundError:
                pass
        else:
            open(file, "wt").write("")
        clt.restart_clock()

    def toggle_show(self, button, file):
        try:
            corr_file = [self.mute_time, self.mute_date].index(file)
        except ValueError:
            pass
        newstate = button.get_active()
        if newstate:
            try:
                os.remove(file)
            except FileNotFoundError:
                pass
        else:
            open(file, "wt").write("")
        try:
            if corr_file == 0:
                self.set_timestate()
            elif corr_file == 1:
                self.set_datestate()
        except UnboundLocalError:
            pass
        clt.restart_clock()

    def set_css(self, hexcol):
        provider = Gtk.CssProvider.new()
        provider.load_from_data(
            css_data.replace("hexcolor", hexcol).encode()
        )
        return provider

    def color_button(self, button, hexcol):
        provider = self.set_css(hexcol)
        color_cont = button.get_style_context()
        color_cont.add_class("colorbutton")
        Gtk.StyleContext.add_provider(
            color_cont,
            provider,
            Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION,
        )

    def update_color(self, *args):
        self.tcolor = clt.hexcolor(clt.read_color(self.tcolorfile))
        self.dcolor = clt.hexcolor(clt.read_color(self.dcolorfile))
        self.color_button(self.d_color, self.dcolor)
        self.color_button(self.t_color, self.tcolor)

    def pick_color(self, button, f):
        wdata = clt.get(["wmctrl", "-l"])
        if "ShowTime - set color" not in wdata:
            subprocess = Gio.Subprocess.new([colorpicker, f], 0)
            subprocess.wait_check_async(None, self.update_color)
            self.update_color()


class BudgieShowTimeApplet(Budgie.Applet):
    """ Budgie.Applet is in fact a Gtk.Bin """

    def __init__(self, uuid):
        Budgie.Applet.__init__(self)
        self.uuid = uuid
        clt.restart_clock()

    def do_get_settings_ui(self):
        """Return the applet settings with given uuid"""
        return BudgieShowTimeSettings(self.get_applet_settings(self.uuid))

    def do_supports_settings(self):
        """Return True if support setting through Budgie Setting,
        False otherwise.
        """
        return True



import gi.repository

gi.require_version('Budgie', '1.0')
from gi.repository import Budgie, GObject, Gtk, Gio
import subprocess
import os
import wprviews_tools as pv
import set_keys

"""
Budgie WindowPreviews
Author: Jacob Vlijm
Copyright © 2017-2019 Ubuntu Budgie Developers
Website=https://ubuntubudgie.org
This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or any later version. This
program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details. You
should have received a copy of the GNU General Public License along with this
program.  If not, see <https://www.gnu.org/licenses/>.
"""

message = """
The following shortcuts are
automatically set:

- Alt + Tab
  (show all windows)
- Alt + Above_Tab (grave)
  (show windows of the
  active application)

Applet runs without
a panel icon
"""

# user
user = pv.user
# plugin path
plugin_path = pv.plugin_path
# panelrunner (wrapper to take care of toggle applet and manage shortcuts)
panelrunner = os.path.join(plugin_path, "wprviews_panelrunner")
# backgrounder, maintaining the set of window prevews
backgrounder = os.path.join(plugin_path, "wprviews_backgrounder")
# settings path
settings_dir = os.path.join(
    os.environ["HOME"], ".config", "budgie-extras", "previews"
)
# file to trigger enabled/disabled from the panelrunner
previews_ismuted = pv.previews_ismuted

# make sure the settings dir exist
try:
    os.makedirs(settings_dir)
except FileExistsError:
    pass


def check_runs(pname):
    # get the pid of a proc
    try:
        pid = subprocess.check_output([
            "pgrep", "-f", "-u", user, pname,
        ]).decode("utf-8")
    except subprocess.CalledProcessError:
        return None
    else:
        return pid.strip()


class WPrviews(GObject.GObject, Budgie.Plugin):
    """ This is simply an entry point into your Budgie Applet implementation.
        Note you must always override Object, and implement Plugin.
    """

    # Good manners, make sure we have unique name in GObject type system
    __gtype_name__ = "WPrviews"

    def __init__(self):
        """ Initialisation is important.
        """
        GObject.Object.__init__(self)

    def do_get_panel_widget(self, uuid):
        """ This is where the real fun happens. Return a new Budgie.Applet
            instance with the given UUID. The UUID is determined by the
            BudgiePanelManager, and is used for lifetime tracking.
        """
        return WPrviewsApplet(uuid)


class WPrviewsSettings(Gtk.Grid):
    keybind = GObject.property(type=bool, default=False)

    def __init__(self, setting):

        super().__init__()

        self.setting = setting
        ismuted = os.path.exists(previews_ismuted)
        # grid & layout
        self.toggle = Gtk.CheckButton.new_with_label("Run WindowPreviews")
        self.toggle.set_active(not ismuted)
        self.toggle.connect("toggled", self.switch)

        self.attach(self.toggle, 0, 0, 1, 1)
        keybinding = Gtk.CheckButton.new_with_label(
            "Disable keyboard shortcuts"
        )
        self.attach(keybinding, 0, 1, 1, 1)
        self.settings = pv.shortc_settings
        self.settings.bind(
            "keybind", self, 'keybind', Gio.SettingsBindFlags.DEFAULT
        )
        keybinding.set_active(self.keybind)
        keybinding.connect("toggled", self.keybind_toggled)

        self.explanation = None
        if not self.keybind:
            self.explanation = Gtk.Label(message)
            self.attach(self.explanation, 0, 2, 1, 1)

        self.show_all()

    def switch(self, button, *args):
        # toggle ui & manage trigger file (noticed by panelrunner)
        pids = check_runs(backgrounder)
        if pids:
            open(previews_ismuted, "wt").write("")
        else:
            subprocess.Popen(panelrunner)
            try:
                os.remove(previews_ismuted)
            except FileNotFoundError:
                pass

    def keybind_toggled(self, button, *args):
        # keybinding ui (noticed by panelrunner)
        self.keybind = not self.keybind
        if self.keybind:
            set_keys.change_keys('restore')
        else:
            set_keys.change_keys('set_custom')

        if not self.explanation:
            self.explanation = Gtk.Label(message)
            self.explanation.show()
            self.attach(self.explanation, 0, 2, 1, 1)

        self.explanation.set_visible(not self.keybind)


class WPrviewsApplet(Budgie.Applet):
    """ Budgie.Applet is in fact a Gtk.Bin """
    manager = None

    def __init__(self, uuid):
        Budgie.Applet.__init__(self)
        self.uuid = uuid
        ismuted = os.path.exists(previews_ismuted)
        if not ismuted:
            self.initiate()

    def initiate(self):
        pids = check_runs(backgrounder)
        if not pids:
            subprocess.Popen(panelrunner)

    def do_get_settings_ui(self):
        """Return the applet settings with given uuid"""
        return WPrviewsSettings(self.get_applet_settings(self.uuid))

    def do_supports_settings(self):
        """Return True if support setting through Budgie Setting,
        False otherwise.
        """
        return True



#!/usr/bin/env python3

"""
Workspace Overview
Author: Jacob Vlijm
Copyright © 2017-2019 Ubuntu Budgie Developers
Website: https://ubuntubudgie.org
This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or any later version. This
program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details. You
should have received a copy of the GNU General Public License along with this
program.  If not, see <http://www.gnu.org/licenses/>.
"""

import gi

gi.require_version('Budgie', '1.0')
gi.require_version('Gtk', '3.0')
from gi.repository import Budgie, GObject, Gtk, Gio
from threading import Thread
import time
import wsotools as wtls
import subprocess

modes = ["Simple", "Dotted", "Elipsed dotted", "Menu"]


class WsOverviewWin(GObject.Object):
    mode_index = GObject.property(type=int, default=2)

    def __init__(self):

        GObject.Object.__init__(self)
        settings = Gio.Settings.new("org.ubuntubudgie.plugins.budgie-extras")
        settings.bind("ws-overview-index", self,
                      'mode_index',
                      Gio.SettingsBindFlags.DEFAULT)
        # general
        self.mode = modes[self.mode_index]

        self.appbutton = Gtk.Button.new()
        self.appbutton.set_relief(Gtk.ReliefStyle.NONE)

        icon = Gtk.Image.new_from_icon_name("ws1-symbolic", Gtk.IconSize.MENU)
        self.appbutton.set_image(icon)

        self.menu = Gtk.Menu()
        self.create_menu()
        self.update = Thread(target=self.show_seconds)
        # daemonize the thread to make the indicator stopable
        self.update.setDaemon(True)
        self.update.start()

    def create_menu(self):
        message = Gtk.MenuItem('Starting up...')
        self.menu.append(message)
        self.menu.show_all()
        self.popup = self.menu
        self.appbutton.connect('clicked', self.popup_menu)

    def edit_menu(self):
        for i in self.menu.get_children():
            self.menu.remove(i)
        for m in self.newmenu:
            add = Gtk.MenuItem(m)
            add.connect('activate', self.get_choice)
            self.menu.append(add)
        # fake separator
        self.menu.append(Gtk.MenuItem(''))
        newspace = Gtk.MenuItem('+')
        newspace.connect('activate', self.add_space)
        self.menu.append(newspace)
        self.change_onthefly()
        self.menu.show_all()

    def edit_menu2(self):
        for i in self.menu.get_children():
            self.menu.remove(i)
        for m in self.newmenu:
            ws = str(m[0] + 1)
            space = Gtk.MenuItem(ws)
            self.menu.append(space)
            if m[1]:
                # flattened submenu
                self.submenu = Gtk.Menu()
                for l in [d for d in m[1]]:
                    app = l[0]
                    wins = [[it[0], it[1]] for it in l[1]]
                    for w in wins:
                        name = self.shortname(w[0]) + "  -  " + app
                        winmention = Gtk.MenuItem(name)
                        self.submenu.append(winmention)
                        winmention.connect('activate', self.move_to, w[1])
                space.set_submenu(self.submenu)
            else:
                space.connect('activate', self.get_choice)
        # fake separator
        self.menu.append(Gtk.MenuItem(''))
        newspace = Gtk.MenuItem('+')
        newspace.connect('activate', self.add_space)
        self.menu.append(newspace)
        self.change_onthefly()
        self.menu.show_all()

    def change_onthefly(self):
        modesep = Gtk.SeparatorMenuItem()
        self.menu.add(modesep)
        mode_mention = Gtk.MenuItem("Mode")
        applet_modes = Gtk.Menu()
        active = modes.index(self.mode)
        self.mode_index = active
        self.menulist = ["\t" + m for m in modes]
        self.menulist[active] = "⁕\t" + str(modes[active]) + ""
        for item in self.menulist:
            md = Gtk.MenuItem(item)
            md.connect('activate', self.set_mode, item)
            applet_modes.append(md)
        mode_mention.set_submenu(applet_modes)
        self.menu.add(mode_mention)

    def set_mode(self, widget, arg):
        self.mode = modes[self.menulist.index(arg)]

    def shortname(self, name):
        """shorten too long names for the menu"""
        limit = 35
        return name[:limit - 3] + "..." if len(name) >= limit else name

    def get_choice(self, mention, *args):
        # move to selected workspace
        index = self.menu.get_children().index(self.menu.get_active())
        subprocess.Popen(["wmctrl", "-s", str(index)])

    def move_to(self, button, wid):
        subprocess.Popen(["wmctrl", "-ia", wid])

    def add_space(self, *args):
        # add one workspace
        settings = Gio.Settings.new("org.gnome.desktop.wm.preferences")
        wkspace = settings["num-workspaces"]
        new_n = wkspace + 1
        settings.set_int("num-workspaces", new_n)

    def show_seconds(self):
        wsdata1, n_ws, curr_ws, curr_windata = None, None, None, None
        update_menu, update_menuset, set_icon = False, False, False
        menuset = []
        # cycle time (sec) for update windowlist
        c2 = 3
        t2 = 0
        self.mode1 = self.mode

        while True:
            # master cycle = 1 sec
            time.sleep(1)
            # test_1: windowlist (cycle2 = 3 sec)
            if all([self.mode in modes[0:3], t2 == 0]):
                # see if mode has changed, act if needed
                self.mode2 = self.mode
                if self.mode2 != self.mode1:
                    update_menu = True
                    update_menuset = True
                    self.mode1 = self.mode2
                """see if data on windowslist changed, if not,
                don't rebuild menudata
                """
                new_windata = wtls.get(["wmctrl", "-l"])
                if all([new_windata, new_windata != curr_windata]):
                    update_menu, update_menuset = True, True
                    curr_windata = new_windata
            elif t2 == 0:
                new_windata = wtls.update_winmenu(curr_windata)
                if new_windata != curr_windata:
                    update_menu, update_menuset = True, True
                    curr_windata = new_windata
            t2 = 0 if t2 == c2 - 1 else t2 + 1
            # test_2: workspace- changes (master_cycle)
            wsdata2 = wtls.get(["wmctrl", "-d"])
            if wsdata2 != wsdata1:
                sp = wtls.getspaces(wsdata2)
                sp0 = sp[0]
                sp1 = sp[1]
                if sp0 != n_ws:
                    update_menu = True
                    n_ws = sp0
                if sp1 != curr_ws:
                    set_icon = True
                    curr_ws = sp1
                wsdata1 = wsdata2
            # apply possible results
            if set_icon:
                newic = wtls.new_icon(curr_ws)
                icon = Gtk.Image.new_from_icon_name(newic, Gtk.IconSize.MENU)
                GObject.idle_add(
                    self.appbutton.set_image, icon,
                    priority=GObject.PRIORITY_DEFAULT
                )
            if self.mode == "Menu":
                if update_menuset:
                    # update window-contents for menu
                    menuset = new_windata[1]
                if update_menu:
                    self.newmenu = []
                    for n in range(n_ws):
                        try:
                            appdata = [
                                applist[1:] for applist in menuset
                                if applist[0] == n
                            ][0]
                            self.newmenu.append([n, appdata])
                        except IndexError:
                            self.newmenu.append([n, None])
                    GObject.idle_add(
                        self.edit_menu2,
                        priority=GObject.PRIORITY_DEFAULT
                    )
            else:
                if update_menuset:
                    # update window-contents for menu
                    menuset = wtls.get_menuset(curr_windata)
                if update_menu:
                    if self.mode == "Dotted":
                        self.newmenu = [" ".join([
                            str(n + 1), menuset.count(str(n)) * "•"
                        ]) for n in range(n_ws)]
                    # limit dots to 3
                    elif self.mode == "Elipsed dotted":
                        self.newmenu = []
                        for n in range(n_ws):
                            n_dots = menuset.count(str(n))
                            n_dots = n_dots if n_dots <= 3 else 3
                            self.newmenu.append(" ".join([
                                str(n + 1), n_dots * "•"
                            ]))
                    if self.mode == "Simple":
                        self.newmenu = [str(n + 1) for n in range(n_ws)]
                        pass
                    GObject.idle_add(
                        self.edit_menu,
                        priority=GObject.PRIORITY_DEFAULT
                    )
                    # reset all
            set_icon, update_menuset, update_menu = False, False, False

    def popup_menu(self, *args):
        self.popup.popup(None, None, None, None, 0,
                         Gtk.get_current_event_time())


class WsOverview(GObject.GObject, Budgie.Plugin):
    """ This is simply an entry point into your Budgie Applet implementation.
        Note you must always override Object, and implement Plugin.
    """

    # Good manners, make sure we have unique name in GObject type system
    __gtype_name__ = "WsOverview"

    def __init__(self):
        """ Initialisation is important.
        """
        GObject.Object.__init__(self)

    def do_get_panel_widget(self, uuid):
        """ This is where the real fun happens. Return a new Budgie.Applet
            instance with the given UUID. The UUID is determined by the
            BudgiePanelManager, and is used for lifetime tracking.
        """
        return WsOverviewApplet(uuid)


class WsOverviewApplet(Budgie.Applet):
    """ Budgie.Applet is in fact a Gtk.Bin """

    # manager = None

    def __init__(self, uuid):
        Budgie.Applet.__init__(self)
        self.app = WsOverviewWin()
        GObject.threads_init()
        self.button = self.app.appbutton
        self.add(self.button)
        self.show_all()



#!/usr/bin/env python3
import os
import subprocess
import time
import gi
gi.require_version('Pango', '1.0')
gi.require_version('Gdk', '3.0')
from gi.repository import Pango, Gdk

"""
Budgie ShowTime
Author: Jacob Vlijm
Copyright © 2017-2019 Ubuntu Budgie Developers
Website=https://ubuntubudgie.org
This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or any later version. This
program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details. You
should have received a copy of the GNU General Public License along with this
program.  If not, see <http://www.gnu.org/licenses/>.
"""

# paths
prefspath = os.path.join(
    os.environ["HOME"], ".config", "budgie-extras", "showtime"
)

app_path = os.path.dirname(os.path.abspath(__file__))
user = os.environ["USER"]


try:
    os.makedirs(prefspath)
except FileExistsError:
    pass

# files
timecolor = os.path.join(prefspath, "timecolor")
datecolor = os.path.join(prefspath, "datecolor")
mute_time = os.path.join(prefspath, "mute_time")
mute_date = os.path.join(prefspath, "mute_date")
pos_file = os.path.join(prefspath, "position")
twelve_hrs = os.path.join(prefspath, "twelve_hrs")
clock = os.path.join(app_path, "ShowTime")
panelrunner = os.path.join(app_path, "bshowtime_panelrunner")


def get(command):
    try:
        return subprocess.check_output(command).decode("utf-8").strip()
    except subprocess.CalledProcessError:
        pass


def get_pid(proc):
    return get(["pgrep", "-f", "-u", user, proc]).splitlines()


def restart_clock():
    for proc in [clock, panelrunner]:
        try:
            for p in get_pid(proc):
                subprocess.Popen(["kill", p])
        except AttributeError:
            pass
    subprocess.Popen(panelrunner)


def get_area():
    # width of the primary screen.
    dspl = Gdk.Display()
    dsp = dspl.get_default()
    prim = dsp.get_primary_monitor()
    geo = prim.get_geometry()
    return [geo.width, geo.height]


def get_textposition():
    try:
        pos = [int(p) for p in open(pos_file).readlines()][:2]
        x = pos[0]
        y = pos[1]
        custom = True
    except (FileNotFoundError, ValueError, IndexError):
        scr = get_area()
        x = (scr[0] * 0.75) - 100
        y = scr[1] * 0.75
        custom = False
    return (custom, x, y)


def read_color(f):
    try:
        return [int(n) for n in open(f).read().splitlines()]
    except FileNotFoundError:
        return [65535, 65535, 65535]


def write_settings(file, newval):
    subj = os.path.join(prefspath, file)
    open(subj, "wt").write(newval)


def hexcolor(rgb):
    c = [int((int(n) / 65535) * 255) for n in rgb]
    return '#%02x%02x%02x' % (c[0], c[1], c[2])


def get_font():
    key = ["org.gnome.desktop.wm.preferences", "titlebar-font"]
    fontdata = get(["gsettings", "get", key[0], key[1]]).strip("'")
    fdscr = Pango.FontDescription(fontdata)
    return Pango.FontDescription.get_family(fdscr)



#!/usr/bin/env python3
import gi
gi.require_version("Gtk", "3.0")
gi.require_version('Budgie', '1.0')
from gi.repository import Budgie, GObject, Gtk, Gio, GdkPixbuf
import os
import dropby_tools as db
import subprocess
import psutil
from threading import Thread


"""
DropBy
Author: Jacob Vlijm
Copyright © 2017-2019 Ubuntu Budgie Developers
Website=https://ubuntubudgie.org
This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or any later version. This
program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details. You
should have received a copy of the GNU General Public License along with this
program.  If not, see <https://www.gnu.org/licenses/>.
"""


css_data = """
.label {
  padding-bottom: 7px;
  padding-top: 0px;
  font-weight: bold;
}
"""


app_path = os.path.dirname(os.path.abspath(__file__))
copyscript = os.path.join(app_path, "copy_flash")


class BudgieDropBy(GObject.GObject, Budgie.Plugin):
    """ This is simply an entry point into your Budgie Applet implementation.
        Note you must always override Object, and implement Plugin.
    """

    # Good manners, make sure we have unique name in GObject type system
    __gtype_name__ = "BudgieDropBy"

    def __init__(self):
        """ Initialisation is important.
        """
        GObject.Object.__init__(self)

    def do_get_panel_widget(self, uuid):
        """ This is where the real fun happens. Return a new Budgie.Applet
            instance with the given UUID. The UUID is determined by the
            BudgiePanelManager, and is used for lifetime tracking.
        """
        return BudgieDropByApplet(uuid)


class ClockWorksSettings(Gtk.Grid):

    def __init__(self, setting):

        super().__init__()
        explanation = Gtk.Label(
            "\n\nThe applet icon will appear when a USB device is connected.",
            xalign=0,
        )
        explanation.set_line_wrap(True)
        self.attach(explanation, 0, 0, 1, 1)
        self.show_all()


class BudgieDropByApplet(Budgie.Applet):
    """ Budgie.Applet is in fact a Gtk.Bin """
    manager = None

    def __init__(self, uuid):
        Budgie.Applet.__init__(self)
        self.uuid = uuid
        self.connect("destroy", Gtk.main_quit)
        self.box = Gtk.EventBox()
        self.icon = Gtk.Image.new_from_icon_name(
            "budgie-dropby-symbolic", Gtk.IconSize.MENU
        )
        self.idle_icon = Gtk.Image.new_from_icon_name(
            "budgie-dropby-idle", Gtk.IconSize.MENU
        )
        self.provider = Gtk.CssProvider.new()
        self.provider.load_from_data(css_data.encode())
        self.box.add(self.icon)
        self.add(self.box)
        self.popover = Budgie.Popover.new(self.box)
        # grid to contain all the stuff
        self.maingrid = Gtk.Grid()
        # throw it in popover
        self.popover.add(self.maingrid)
        self.popover.get_child().show_all()
        self.box.show_all()
        self.show_all()
        self.box.connect("button-press-event", self.on_press)
        # thread
        GObject.threads_init()
        self.update = Thread(target=self.setup_watching)
        # daemonize the thread to make the indicator stopable
        self.update.setDaemon(True)
        self.update.start()
        self.refresh_from_idle()

    def setup_watching(self):
        self.watchdrives = Gio.VolumeMonitor.get()
        self.triggers = [
            "volume_added", "volume_removed", "mount_added", "mount_removed",
        ]
        for t in self.triggers:
            self.watchdrives.connect(t, self.refresh_from_idle)
        # workaround to only open nautilus on our own action
        self.act_onmount = False
        # make the applet pop up on the event of a new volume
        self.watchdrives.connect("volume_added", self.on_event, self.box)

    def do_get_settings_ui(self):
        """Return the applet settings with given uuid"""
        return ClockWorksSettings(self.get_applet_settings(self.uuid))

    def do_supports_settings(self):
        """Return True if support setting through Budgie Setting,
        False otherwise.
        """
        return True

    def fill_grid(self, get_relevant, newvol):
        pos = 2
        for d in get_relevant:
            # get icon
            icon = Gtk.Image.new_from_gicon(d["icon"], Gtk.IconSize.MENU)
            # get name
            vol_name = d["name"]
            # mark possible new volume
            addition = " *" if d["volume"] == newvol else ""
            namebutton = Gtk.Button(" " + vol_name + addition, xalign=0)
            namebutton.set_image(icon)
            namebutton.set_always_show_image(True)
            namebutton.set_relief(Gtk.ReliefStyle.NONE)
            self.maingrid.attach(namebutton, 2, pos, 1, 1)
            # show free space
            freespace = Gtk.Label("     " + d["free"], xalign=0)
            self.maingrid.attach(freespace, 3, pos, 1, 1)
            # set gui attributes for mounted volumes
            if d["ismounted"]:
                mount = d["ismounted"]
                vol_path = d["volume_path"]
                if all([mount == newvol, self.act_onmount]):
                    self.open_folder(vol_path)
                    self.act_onmount = False
                eject_button = Gtk.Button()
                eject_button.set_relief(Gtk.ReliefStyle.NONE)
                eject_button.set_image(Gtk.Image.new_from_icon_name(
                    "media-eject-symbolic", Gtk.IconSize.MENU))
                self.maingrid.attach(eject_button, 6, pos, 1, 1)
                if d["flashdrive"]:
                    spacer = Gtk.Label("    ")
                    self.maingrid.attach(spacer, 4, pos, 1, 1)
                    tooltip = "Eject"
                    eject_button.connect("clicked", self.eject_volume, mount)
                    cp_button = Gtk.Button()
                    cp_button.set_image(Gtk.Image.new_from_icon_name(
                        "media-floppy-symbolic", Gtk.IconSize.MENU))
                    cp_button.set_relief(Gtk.ReliefStyle.NONE)
                    cp_button.set_tooltip_text("Make a local copy")
                    cp_button.connect(
                        "clicked", self.copy_flashdrive, vol_path, vol_name,
                    )
                    self.maingrid.attach(cp_button, 5, pos, 1, 1)
                else:
                    tooltip = "Unmount"
                    eject_button.connect("clicked", self.unmount_volume, mount)
                eject_button.set_tooltip_text(tooltip)
                namebutton.set_tooltip_text("Open " + vol_path)
                namebutton.connect("clicked", self.open_folder, vol_path)
            else:
                namebutton.connect("clicked", self.mount_volume, d["volume"])
                tooltip = "Mount and open " + vol_name
                namebutton.set_tooltip_text(tooltip)
            pos = pos + 1
        # create headers
        volume_label = Gtk.Label("  Volume", xalign=0)
        freespace_label = Gtk.Label("    Free   ", xalign=0)
        self.maingrid.attach(volume_label, 2, 1, 1, 1)
        self.maingrid.attach(freespace_label, 3, 1, 1, 1)
        for label in [volume_label, freespace_label]:
            label_cont = label.get_style_context()
            label_cont.add_class("label")
            Gtk.StyleContext.add_provider(
                label_cont,
                self.provider,
                Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION,
            )
        self.set_spacers()

    def refresh_from_idle(self, subject=None, newvol=None):
        GObject.idle_add(
            self.refresh, subject, newvol,
            priority=GObject.PRIORITY_DEFAULT,
        )

    def refresh(self, subject=None, newvol=None):
        # empty grid
        for c in self.maingrid.get_children():
            c.destroy()
        # lookup usb drives
        allvols = self.watchdrives.get_volumes()
        get_relevant = db.get_volumes(allvols)
        # decide if we should show or not
        for c in self.box.get_children():
            c.destroy()
        if get_relevant:
            self.box.add(self.icon)
            self.fill_grid(get_relevant, newvol)
        else:
            self.box.add(self.idle_icon)
        self.box.show_all()

    def open_folder(self, *args):
        path = list(args)[-1]
        subprocess.Popen(["xdg-open", path])

    def mount_volume(self, button, vol):
        Gio.Volume.mount(
            vol, Gio.MountMountFlags.NONE, Gio.MountOperation(), None
        )
        self.act_onmount = True

    def unmount_volume(self, button, vol):
        Gio.Mount.unmount(vol, Gio.MountUnmountFlags.NONE, None)

    def eject_volume(self, button, vol):
        Gio.Mount.eject(vol, Gio.MountUnmountFlags.NONE, None)

    def copy_flashdrive(self, button, source, name):
        subprocess.Popen([copyscript, source, name])

    def set_spacers(self):
        """
        lazy choice to set borders
        """
        c = [[0, 0], [100, 0], [0, 100], [100, 100]]
        for n in range(4):
            spacer = Gtk.Label(" " * 8)
            self.maingrid.attach(spacer, c[n][0], c[n][1], 1, 1)
        self.maingrid.show_all()

    def lockscreen_check(self):
        lockproc = "gnome-screensaver-dialog"
        try:
            return lockproc in (p.name() for p in psutil.process_iter())
        except psutil.NoSuchProcess:
            return False

    def scrs_active_check(self):
        cmd = ["gnome-screensaver-command", "-t"]
        try:
            output = subprocess.check_output(cmd).decode("utf-8")
            return not any(char.isdigit() for char in output)
        except Exception:
            return False

    def on_event(self, box, *args):
        if all([
            not self.lockscreen_check(), self.scrs_active_check()
        ]):
            GObject.idle_add(
                self.manager.show_popover, self.box,
                priority=GObject.PRIORITY_DEFAULT,
            )

    def on_press(self, box, arg):
        self.refresh_from_idle()
        self.manager.show_popover(self.box)

    def do_update_popovers(self, manager):
        self.manager = manager
        self.manager.register_popover(self.box, self.popover)



#!/usr/bin/python3

# This file is part of App Launcher

# Copyright © 2018-2019 Ubuntu Budgie Developers

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.

import gi.repository

gi.require_version('Gtk', '3.0')
from gi.repository import Gtk


class SelectButton(Gtk.Button):

    def __init__(self, active):
        Gtk.Button.__init__(self)

        self.active = active
        self.SELECT_ALL = "Select all"
        self.DESELECT_ALL = "Deselect all"
        self.label = Gtk.Label()
        self.add(self.label)
        self.set_size_request(96, 0)
        self.setActive(self.active)
        self.get_style_context().add_class("flat")

    def setActive(self, active):
        self.active = active
        if active:
            markup = "<span size='small'>%s</span>" % \
                     self.SELECT_ALL.replace("&", "&amp;")
        else:
            markup = "<span size='small'>%s</span>" % \
                     self.DESELECT_ALL.replace("&", "&amp;")
        self.label.set_markup(markup)

    def isActive(self):
        return self.active

    def addOnClickMethod(self, method):
        self.connect("clicked", method)

    def setSensitive(self, isSensitive):
        self.set_sensitive(isSensitive)



#!/usr/bin/env python3
import svgwrite
from cairosvg import svg2png
import os
from PIL import Image
import gi
from gi.repository import Gio
import subprocess

"""
ClockWorks
Author: Jacob Vlijm
Copyright © 2017-2019 Ubuntu Budgie Developers
Website=https://ubuntubudgie.org
This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or any later version. This
program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details. You
should have received a copy of the GNU General Public License along with this
program.  If not, see <https://www.gnu.org/licenses/>.
"""

tz_data = [
    "-12:00", "-11:00", "-10:00", "-09:30", "-09:00", "-08:00", "-07:00",
    "-06:00", "-05:00", "-04:00", "-03:30", "-03:00", "-02:00", "-01:00",
    "00:00", "+01:00", "+02:00", "+03:00", "+03:30", "+04:00", "+04:30",
    "+05:00", "+05:30", "+05:45", "+06:00", "+06:30", "+07:00", "+08:00",
    "+08:30", "+08:45", "+09:00", "+09:30", "+10:00", "+10:30", "+11:00",
    "+12:00", "+12:45", "+13:00", "+14:00",
]


def get(command):
    try:
        return subprocess.check_output(command).decode("utf-8").strip()
    except subprocess.CalledProcessError:
        pass


# dirs etc.
home = os.environ["HOME"]
settingsdir = os.path.join(home, ".config/budgie-extras/clockworks")
hrs_path = os.path.join(settingsdir, "hrs")
mins_path = os.path.join(settingsdir, "mins")
misc_dir = os.path.join(settingsdir, "misc")
user = os.environ["USER"]
tmp = os.path.join("/tmp", user + "_clockworks")
clock_datafile = os.path.join(settingsdir, "clockdata")
key = "org.ubuntubudgie.plugins.budgie-clockworks"
subkeys = ["background", "hour", "minute"]
settings = Gio.Settings.new(key)


# make sure directories exist
for dr in [hrs_path, mins_path, misc_dir, tmp]:
    try:
        os.makedirs(dr)
    except FileExistsError:
        pass


def prepare_rgb(hx):
    return [int(hx.strip("#")[i:i + 2], 16) for i in (0, 2, 4)]


def hex2rgb(hx):
    # convert hex color to rgb
    return "rgb(" + ", ".join([str(n) for n in prepare_rgb(hx)]) + ")"


def rgb2hex(r, g, b):
    hx = "#{:02x}{:02x}{:02x}".format(r, g, b)
    return hx


def get_current_colors():
    # produce current gsettings values
    return [
        settings.get_string(val) for val in [
            "background", "hour", "minute"
        ]
    ]


def create_png(svg, target):
    lines = open(svg).readlines()[1:][0]
    svg2png(bytestring=lines, write_to=target)


def create_bg(hx):
    # temp file
    tmp_bg = os.path.join(tmp, "tmp_bg.svg")
    # create svg file
    bg = svgwrite.Drawing(filename=tmp_bg, size=("100px", "100px"))
    # define circle
    bg.add(
        bg.circle(
            center=(50, 50), r=48, stroke_width="4",
            stroke=hex2rgb(hx), fill="none",
        )
    )
    # save & convert
    bg.save()
    create_png(tmp_bg, os.path.join(misc_dir, "background_image.png"))


def save_togsettings(color, subkey):
    settings.set_string(subkey, color)


def create_minutes(hx):
    tmp_min = os.path.join(tmp, "tmp_min.svg")
    minute = svgwrite.Drawing(
        filename=tmp_min, size=("100px", "100px")
    )
    minute.add(minute.line(
        start=(50, 45), end=(50, 10), stroke_width="3",
        stroke=hex2rgb(hx),
    )
    )
    minute.save()
    temp_minspng = tmp_min.replace(".svg", ".png")
    create_png(tmp_min, temp_minspng)
    # minutes
    min_source = Image.open(temp_minspng)
    for n in range(60):
        rotate = n * 6
        newminspath = os.path.join(mins_path, str(n) + ".png")
        new_min = min_source.rotate(
            rotate, resample=Image.BICUBIC, expand=False
        )
        new_min.save(newminspath)


def create_hours(hx):
    tmp_hr = os.path.join(tmp, "tmp_hr.svg")
    hr = svgwrite.Drawing(
        filename=tmp_hr, size=("100px", "100px")
    )
    hr.add(hr.line(
        start=(50, 45), end=(50, 20), stroke_width="3",
        stroke=hex2rgb(hx),
    )
    )
    hr.save()
    temp_hrspng = tmp_hr.replace(".svg", ".png")
    create_png(tmp_hr, temp_hrspng)
    # hours
    hr_source = Image.open(temp_hrspng)
    for n in range(60):
        rotate = n * 6
        newhrpath = os.path.join(hrs_path, str(n) + ".png")
        new_hr = hr_source.rotate(
            rotate, resample=Image.BICUBIC, expand=False
        )
        new_hr.save(newhrpath)


def create_set():
    curr_settings = get_current_colors()
    create_bg(curr_settings[0])
    create_hours(curr_settings[1])
    create_minutes(curr_settings[2])



#!/usr/bin/env python3
import os
import subprocess
import gi
gi.require_version('Gdk', '3.0')
from gi.repository import Gdk, Gio


"""
Budgie WindowPreviews
Author: Jacob Vlijm
Copyright © 2017-2019 Ubuntu Budgie Developers
Website=https://ubuntubudgie.org
This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or any later version. This
program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details. You
should have received a copy of the GNU General Public License along with this
program.  If not, see <http://www.gnu.org/licenses/>.
"""

dcpath = "/com/solus-project/budgie-panel/applets/"
plugin_path = os.path.dirname(os.path.abspath(__file__))
shortc_settings = Gio.Settings.new("org.ubuntubudgie.plugins.budgie-wpreviews")


user = os.environ["USER"]
previews_dir = "/tmp"
settings_dir = os.path.join(
    os.environ["HOME"], ".config", "budgie-extras", "previews"
)

previews = os.path.join(
    previews_dir,
    user + "_window-previews",
)

previews_ismuted = os.path.join(settings_dir, "muted")
temp_history = os.path.join(previews_dir, user + "_focus_history")

for dr in [previews_dir, previews, settings_dir]:
    try:
        os.makedirs(dr)
    except FileExistsError:
        pass

ignore = [
    "= _NET_WM_WINDOW_TYPE_DOCK",
    "= _NET_WM_WINDOW_TYPE_DESKTOP",
]


# default resize is v_size, unless w exceeds threshold
max_w = 260
v_size = 160
# strings, to be used in the resize- commands
comm = str(max_w) + "x" + str(v_size)


def getkey():
    # get the specific dconf path, referring to the applet's key
    data = subprocess.check_output([
        "dconf", "dump", dcpath,
    ]).decode("utf-8").splitlines()
    try:
        match = [l for l in data if "Window Previews" in l][0]
        watch = data.index(match) - 3
        return data[watch][1:-1]
    except IndexError:
        pass


def get_area():
    # width of the primary screen.
    dsp = Gdk.Display().get_default()
    prim = dsp.get_primary_monitor()
    return prim.get_geometry().width


def get(cmd):
    # just a helper
    try:
        return subprocess.check_output(cmd).decode("utf-8").strip()
    except (subprocess.CalledProcessError, TypeError, UnicodeDecodeError):
        pass


def get_ws():
    # get current workspace
    try:
        return [l.split()[0] for l in get([
            "wmctrl", "-d"
        ]).splitlines() if "*" in l][0]
    except AttributeError:
        pass


def empty_dir():
    for w in os.listdir(previews):
        path = os.path.join(previews, w)
        os.remove(path)


def get_valid(w_id):
    # see if the window is a valid one (type)
    w_data = get(["xprop", "-id", w_id])
    if w_data:
        return True if not any([t in w_data for t in ignore]) else False
    else:
        return False


def show_wmclass(wid):
    # get WM_CLASS from window- id
    try:
        cl = get(["xprop", "-id", wid, "WM_CLASS"]).split("=")[-1].strip()
    except (IndexError, AttributeError):
        pass
    else:
        # exceptions; one application, multiple WM_CLASS
        if "Thunderbird" in cl:
            return "Thunderbird"
        elif "Toplevel" in cl:
            return "Toplevel"
        else:
            return cl


def get_activeclass():
    # get WM_CLASS of active window
    return show_wmclass(get(["xdotool", "getactivewindow"]))


def get_hex(w_id):
    win = hex(int(w_id))
    return win[:2] + (10 - len(win)) * "0" + win[2:]


def get_wmname(w_id):
    # get WM_NAME from window- id
    return get(["xdotool", "getwindowname", w_id])



import gi.repository
gi.require_version('Budgie', '1.0')
from gi.repository import Budgie, GObject, Gtk
import subprocess
import os
import wmovertools as wmt

"""
Budgie WindowMover
Author: Jacob Vlijm
Copyright © 2017-2019 Ubuntu Budgie Developers
Website=https://ubuntubudgie.org
This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or any later version. This
program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details. You
should have received a copy of the GNU General Public License along with this
program.  If not, see <http://www.gnu.org/licenses/>.
"""

panelrunner = os.path.join(wmt.appletpath, "wmover_panelrunner")
backgrounder = os.path.join(wmt.appletpath, "wmover_run")
wmover_path = wmt.settings_path
wmover_ismuted = wmt.wmover_ismuted
user = wmt.user

try:
    os.makedirs(wmover_path)
except FileExistsError:
    pass


def check_runs(pname):
    # get the pid of a proc
    try:
        pid = subprocess.check_output([
            "pgrep", "-f", "-u", user, pname,
        ]).decode("utf-8")
    except subprocess.CalledProcessError:
        return None
    else:
        return pid.strip()


message = """
The following shortcuts are
automatically set:

- Ctrl + Alt + w
  (call the window mover)
  then press the number of
  the targeted workspace.

- Ctrl + Alt + s
  (call the desktop mover)
  then press the number of
  the targeted workspace.

Or: drag the window to the
bottom of your screen, and the
window mover appears

Applet runs without a panel
icon
"""


class WPrviewsSettings(Gtk.Grid):
    def __init__(self, setting):

        super().__init__()

        self.setting = setting
        ismuted = os.path.exists(wmt.wmover_ismuted)
        # grid & layout
        explanation = Gtk.Label(message)
        self.attach(explanation, 0, 1, 1, 1)
        self.toggle = Gtk.CheckButton.new_with_label(" Run Window Mover")
        self.toggle.set_active(not ismuted)
        self.toggle.connect("toggled", self.switch)
        self.attach(self.toggle, 0, 0, 1, 1)
        self.show_all()

    def switch(self, button, *args):
        # toggle ui & manage trigger file (noticed by panelrunner)
        pids = check_runs(backgrounder)
        if pids:
            open(wmover_ismuted, "wt").write("")
        else:
            subprocess.Popen(panelrunner)
            try:
                os.remove(wmover_ismuted)
            except FileNotFoundError:
                pass


class WMover(GObject.GObject, Budgie.Plugin):
    """ This is simply an entry point into your Budgie Applet implementation.
        Note you must always override Object, and implement Plugin.
    """

    # Good manners, make sure we have unique name in GObject type system
    __gtype_name__ = "WMover"

    def __init__(self):
        """ Initialisation is important.
        """
        GObject.Object.__init__(self)

    def do_get_panel_widget(self, uuid):
        """ This is where the real fun happens. Return a new Budgie.Applet
            instance with the given UUID. The UUID is determined by the
            BudgiePanelManager, and is used for lifetime tracking.
        """
        return WMoverApplet(uuid)


class WMoverApplet(Budgie.Applet):
    """ Budgie.Applet is in fact a Gtk.Bin """
    manager = None

    def __init__(self, uuid):
        Budgie.Applet.__init__(self)
        self.uuid = uuid
        ismuted = os.path.exists(wmt.wmover_ismuted)
        self.box = Gtk.EventBox()
        if not ismuted:
            self.initiate()

    def initiate(self):
        pass
        pids = check_runs(backgrounder)
        if not pids:
            subprocess.Popen(panelrunner)

    def do_get_settings_ui(self):
        """Return the applet settings with given uuid"""
        return WPrviewsSettings(self.get_applet_settings(self.uuid))

    def do_supports_settings(self):
        """Return True if support setting through Budgie Setting,
        False otherwise.
        """
        return True



#!/usr/bin/env python3
import ast
import os
import subprocess

"""
Budgie WindowMover
Author: Jacob Vlijm
Copyright © 2017-2019 Ubuntu Budgie Developers
Website=https://ubuntubudgie.org
This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or any later version. This
program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details. You
should have received a copy of the GNU General Public License along with this
program.  If not, see <http://www.gnu.org/licenses/>.
"""

# the key defining the custom keys
key = [
    "org.gnome.settings-daemon.plugins.media-keys",
    "custom-keybindings",
    "custom-keybinding",
]
# the shortcut names to look up in dconf
shortcut_names = ["wmover_window", "wmover_workspace"]
# command (main-) line to run previews
aw = os.path.dirname(os.path.abspath(__file__)) + "/wmover_run"


def get(cmd):
    return subprocess.check_output(cmd).decode("utf-8").strip()


def get_currnames():
    relevant = []
    allnames = []
    try:
        customs = ast.literal_eval(
            get(["gsettings", "get", key[0], key[1]])
        )
    except SyntaxError:
        return [], []
    else:
        for c in customs:
            name = get([
                "gsettings", "get", key[0] + "." + key[2] + ":" + c, "name",
            ]).replace("'", "")
            if name in shortcut_names:
                relevant.append(c)
            allnames.append(c)
    return allnames, relevant


def remove_custom():
    customs = get_currnames()[0]
    remove = get_currnames()[1]
    newlist = [item for item in customs if item not in remove]
    subprocess.call([
        "gsettings", "set", key[0], key[1], str(newlist),
    ])


def define_keyboard_shortcut(name, command, shortcut):
    # defining keys & strings to be used
    # params example 'open gedit' 'gedit' '<Alt>7'
    subkey1 = ".".join([key[0], key[2]]) + ":"
    item_s = "/" + subkey1[:-1] + "s".replace(".", "/") + "/"
    firstname = "custom"
    # get the current list of custom shortcuts
    getcurrent = get(["gsettings", "get", key[0], key[1]])
    if '@as []' in getcurrent:
        current = []
    else:
        current = ast.literal_eval(getcurrent)
    # make sure the additional keybinding mention is no duplicate
    n = 1
    while True:
        new = item_s + firstname + str(n) + "/"
        if new in current:
            n = n + 1
        else:
            break
    # add the new keybinding to the list
    current.append(new)
    # create the shortcut, set the name, command and shortcut key
    for cmd in ([
        [key[0], key[1], str(current)],
        [subkey1 + new, "name", name],
        [subkey1 + new, "command", command],
        [subkey1 + new, "binding", shortcut],
    ]):
        subprocess.call(["gsettings", "set"] + cmd)


def change_keys(arg):
    # clean up possible duplicates (from unclean stop)
    remove_custom()
    if arg == "set_custom":
        # clean up possible duplicates (from unclean stop)
        define_keyboard_shortcut("wmover_window",
                                 aw + " -single", '<Control><Alt>w')
        define_keyboard_shortcut("wmover_workspace",
                                 aw + " -singlespace", '<Control><Alt>s')
    elif arg == "restore":
        pass



import gi

gi.require_version("Gtk", "3.0")
gi.require_version('Budgie', '1.0')
from gi.repository import Gdk, Gtk, GObject, GdkPixbuf, Budgie, Gio
import os
import time
from threading import Thread
import math
import subprocess
import ast
import gi.repository
import cwtools as cw

"""
ClockWorks
Author: Jacob Vlijm
Copyright © 2017-2019 Ubuntu Budgie Developers
Website=https://ubuntubudgie.org
This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or any later version. This
program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details. You
should have received a copy of the GNU General Public License along with this
program.  If not, see <https://www.gnu.org/licenses/>.
"""

# paths
home = cw.home
settingsdir = cw.settingsdir
hrs_path = cw.hrs_path
mins_path = cw.mins_path
clock_datafile = cw.clock_datafile
app_path = os.path.dirname(os.path.abspath(__file__))

# make sure directories exist
for dr in [hrs_path, mins_path]:
    try:
        os.makedirs(dr)
    except FileExistsError:
        pass

hours = []
minutes = []
tz_data = cw.tz_data
settings = cw.settings

# create images
cw.create_set()

for n in range(60):
    # hours
    newhrpath = os.path.join(hrs_path, str(n) + ".png")
    hours.append(GdkPixbuf.Pixbuf.new_from_file(newhrpath))
    # minutes
    newminspath = os.path.join(mins_path, str(n) + ".png")
    minutes.append(GdkPixbuf.Pixbuf.new_from_file(newminspath))

clw_css_data = """
.label {
  padding-bottom: 5px;
  padding-top: 5px;
  font-weight: bold;
}
"""

clw_colordata = """
.colorbutton {
  border-color: transparent;
  padding: 0px;
  border-width: 10px;
  border-radius: 4px;
}
.colorbutton:hover {
  border-color: transparent;
  padding: 0px;
  border-width: 10px;
  border-radius: 4px;
}
"""


class BudgieClockWorks(GObject.GObject, Budgie.Plugin):
    """ This is simply an entry point into your Budgie Applet implementation.
        Note you must always override Object, and implement Plugin.
    """

    # Good manners, make sure we have unique name in GObject type system
    __gtype_name__ = "BudgieClockWorks"

    def __init__(self):
        """ Initialisation is important.
        """
        GObject.Object.__init__(self)

    def do_get_panel_widget(self, uuid):
        """ This is where the real fun happens. Return a new Budgie.Applet
            instance with the given UUID. The UUID is determined by the
            BudgiePanelManager, and is used for lifetime tracking.
        """
        return BudgieClockWorksApplet(uuid)


class ClockWorksSettings(Gtk.Grid):

    def __init__(self, setting):

        super().__init__()

        # grid & layout
        self.set_row_spacing(12)
        element_hsizer1 = self.h_spacer(13)
        self.attach(element_hsizer1, 0, 0, 1, 7)
        element_hsizer2 = self.h_spacer(25)
        self.attach(element_hsizer2, 2, 0, 1, 7)
        # color buttons & labels
        self.clockworks_colorbuttons = []
        labels = ["Frame color", "Hours color", "Minutes color"]
        self.initial_colors = [
            cw.prepare_rgb(hx) for hx in cw.get_current_colors()
        ]
        for n in range(3):
            # get color
            buttoncolor = self.initial_colors[n]
            color = Gdk.RGBA()
            color.red = buttoncolor[0] / 255
            color.green = buttoncolor[1] / 255
            color.blue = buttoncolor[2] / 255
            color.alpha = 1
            b_container = Gtk.Box()
            self.attach(b_container, 1, 4 + n, 1, 1)
            colorbutton = Gtk.ColorButton()
            colorbutton.set_rgba(color)
            self.set_buttonstyle(colorbutton)
            self.clockworks_colorbuttons.append(colorbutton)
            colorbutton.connect("color-set", self.pick_color, n)
            colorbutton.set_size_request(10, 10)
            b_container.pack_start(colorbutton, False, False, 0)
            label = Gtk.Label("\t" + labels[n])
            b_container.pack_start(label, False, False, 0)
        self.show_all()

    def pick_color(self, button, color_index):
        subs = cw.subkeys
        newcolor = button.get_rgba()
        r = round(newcolor.red * 255)
        g = round(newcolor.green * 255)
        b = round(newcolor.blue * 255)
        hx = cw.rgb2hex(r, g, b)
        cw.save_togsettings(hx, subs[color_index])

    def h_spacer(self, addwidth):
        # horizontal spacer
        spacegrid = Gtk.Grid()
        if addwidth:
            label1 = Gtk.Label()
            label2 = Gtk.Label()
            spacegrid.attach(label1, 0, 0, 1, 1)
            spacegrid.attach(label2, 1, 0, 1, 1)
            spacegrid.set_column_spacing(addwidth)
        return spacegrid

    def set_buttonstyle(self, button):
        provider = Gtk.CssProvider.new()
        provider.load_from_data(
            clw_colordata.encode())
        color_cont = button.get_style_context()
        color_cont.add_class("colorbutton")
        Gtk.StyleContext.add_provider(
            color_cont,
            provider,
            Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION,
        )


class BudgieClockWorksApplet(Budgie.Applet):
    """ Budgie.Applet is in fact a Gtk.Bin """
    manager = None

    def __init__(self, uuid):
        Budgie.Applet.__init__(self)
        self.uuid = uuid
        self.connect("destroy", Gtk.main_quit)
        self.settings = cw.settings
        icon = Gtk.Image.new_from_icon_name(
            "budgie-clockworks-symbolic", Gtk.IconSize.MENU
        )
        self.provider = Gtk.CssProvider.new()
        self.provider.load_from_data(clw_css_data.encode())
        # maingrid
        self.maingrid = Gtk.Grid()
        self.maingrid.set_row_spacing(2)
        self.maingrid.attach(Gtk.Label(" " * 10), 0, 0, 1, 1)
        self.maingrid.set_column_spacing(5)
        self.clocklist = {}
        # create initial clock if it does not exists
        currcl_data = self.read_datafile()
        if currcl_data:
            for cl in currcl_data:
                off = cl[1]
                clname = cl[2]
                self.create_newclock(offset=off, clockname=clname)
        else:
            self.create_newclock(offset=0)
        self.dashbuttonbox = Gtk.Box()
        self.add_button = Gtk.Button()
        self.add_button.set_relief(Gtk.ReliefStyle.NONE)
        self.add_icon = Gtk.Image.new_from_icon_name(
            "list-add-symbolic", Gtk.IconSize.MENU
        )
        self.add_button.set_image(self.add_icon)
        self.search_button = Gtk.Button()
        self.search_button.set_relief(Gtk.ReliefStyle.NONE)
        self.search_icon = Gtk.Image.new_from_icon_name(
            "system-search-symbolic", Gtk.IconSize.MENU
        )
        self.search_button.set_image(self.search_icon)
        self.add_button.connect("clicked", self.create_newclock)
        self.search_button.connect("clicked", self.run_search)
        self.maingrid.attach(self.add_button, 100, 3, 1, 1)
        self.maingrid.attach(self.search_button, 100, 4, 1, 1)
        # throw it in popover
        self.box = Gtk.EventBox()
        self.box.add(icon)
        self.add(self.box)
        self.popover = Budgie.Popover.new(self.box)
        self.popover.add(self.maingrid)
        self.popover.get_child().show_all()
        self.box.show_all()
        self.show_all()
        self.box.connect("button-press-event", self.on_press)
        self.settings.connect("changed", self.update_clockcolor)
        # thread
        GObject.threads_init()
        self.update = Thread(target=self.update_gmt)
        # daemonize the thread to make the indicator stopable
        self.update.setDaemon(True)
        self.update.start()

    def do_get_settings_ui(self):
        """Return the applet settings with given uuid"""
        return ClockWorksSettings(self.get_applet_settings(self.uuid))

    def update_clockcolor(self, key, sub):
        subs = cw.subkeys
        newcolor = settings.get_string(subs[subs.index(sub)])
        if sub == "background":
            cw.create_bg(newcolor)
            keys = list(self.clocklist.keys())
            for k in keys:
                newbg = Gtk.Image.new_from_file(
                    os.path.join(cw.misc_dir, "background_image.png")
                )
                old = self.clocklist[k]["misc_widgets"][0]
                old.destroy()
                self.maingrid.attach(newbg, k, 1, 1, 5)
                self.clocklist[k]["misc_widgets"][0] = newbg
            self.maingrid.show_all()
        elif sub == "hour":
            cw.create_hours(newcolor)
            for n in range(60):
                newhrpath = os.path.join(hrs_path, str(n) + ".png")
                hours.append(GdkPixbuf.Pixbuf.new_from_file(newhrpath))
            self.refresh_clocks()
        elif sub == "minute":
            cw.create_minutes(newcolor)
            for n in range(60):
                newminspath = os.path.join(mins_path, str(n) + ".png")
                minutes.append(GdkPixbuf.Pixbuf.new_from_file(newminspath))
            self.refresh_clocks()

    def do_supports_settings(self):
        """Return True if support setting through Budgie Setting,
        False otherwise.
        """
        return True

    def run_search(self, button):
        subprocess.Popen(
            ["xdg-open", "https://www.timeanddate.com/time/map/"]
        )

    def double_digits(self, t):
        digits = len(t)
        return ((2 - digits) * "0") + t if digits < 2 else t

    def convert_offset_tolabel(self, offset):
        if offset != 0:
            prestring = "-" if offset < 0 else "+"
            set_time_hrs = self.double_digits(str(int(abs(offset) / 3600)))
            set_time_mins = self.double_digits(str(int((offset % 3600) / 60)))
            return prestring + set_time_hrs + ":" + set_time_mins
        else:
            return "00:00"

    def convert_label_tooffset(self, label):
        newval = label.split(":")
        hrs_offset = int(newval[0]) * 3600
        mins_offset = int(newval[1]) * 60
        mins_offset = mins_offset if hrs_offset > 0 else -mins_offset
        return (hrs_offset + mins_offset)

    def save_tofile(self):
        store_data = []
        for k in self.clocklist.keys():
            store_data.append([
                k, self.clocklist[k]["offset"],
                self.clocklist[k]["clocklabel"],
            ])
        open(clock_datafile, "wt").write(str(store_data))

    def read_datafile(self):
        try:
            return ast.literal_eval(open(clock_datafile).read().strip())
        except (FileNotFoundError, SyntaxError):
            pass

    def create_newclock(self, *args, offset=0, clockname=None):
        clockkeys = list(self.clocklist.keys())
        try:
            clock_id = max(list(clockkeys)) + 1
        except ValueError:
            clock_id = 1
        # set clock image
        bg = Gtk.Image.new_from_file(
            os.path.join(cw.misc_dir, "background_image.png")
        )
        # initial minute- image
        minute_image = Gtk.Image.new_from_pixbuf(minutes[0])
        # initial hour- imag
        hour_image = Gtk.Image.new_from_pixbuf(hours[0])
        for widget in [bg, minute_image, hour_image]:
            self.maingrid.attach(widget, clock_id, 1, 1, 5)
        # am/pm label
        ampm_label = Gtk.Label(xalign=0.5)
        # delete_clock -box + button
        trashbox = Gtk.Box()
        delete_button = Gtk.Button()
        delete_button.set_image(Gtk.Image.new_from_icon_name(
            "user-trash-symbolic", Gtk.IconSize.MENU)
        )
        delete_button.set_relief(Gtk.ReliefStyle.NONE)
        trashbox.pack_end(delete_button, False, False, 0)
        self.maingrid.attach(trashbox, clock_id, 0, 1, 1)
        self.maingrid.attach(ampm_label, clock_id, 6, 1, 1)
        # timezonbe label, up- down buttons
        offset_down = Gtk.Button()
        offset_down.set_image(
            Gtk.Image.new_from_icon_name(
                "pan-down-symbolic", Gtk.IconSize.MENU
            )
        )
        offset_down.set_relief(Gtk.ReliefStyle.NONE)
        offset_up = Gtk.Button()
        offset_up.set_image(
            Gtk.Image.new_from_icon_name(
                "pan-up-symbolic", Gtk.IconSize.MENU
            )
        )
        offset_up.set_relief(Gtk.ReliefStyle.NONE)
        timezone_time = Gtk.Label(xalign=0.5)
        timezone_time.set_text(self.convert_offset_tolabel(offset))
        timezone_time.set_width_chars(6)
        timezone_box = Gtk.Box(Gtk.BaselinePosition.CENTER)
        timezone_box.set_spacing(0)
        self.maingrid.attach(timezone_box, clock_id, 7, 1, 1)
        timezone_box.pack_start(offset_down, False, False, 0)
        timezone_box.pack_start(timezone_time, False, False, 0)
        timezone_box.set_center_widget(timezone_time)
        timezone_box.pack_start(offset_up, False, False, 0)
        # label entry
        clockname_label = Gtk.Entry()
        clockname_label.set_max_width_chars(15)
        clockname_label.set_width_chars(15)
        clockname_label.set_placeholder_text("Clock label")
        clockname_label.set_alignment(0.5)
        self.maingrid.attach(clockname_label, clock_id, 8, 1, 1)
        if clockname:
            clockname_label.set_text(clockname)
        else:
            clockname = ""
        offset_down.connect("pressed", self.set_down, clock_id, timezone_time)
        offset_up.connect("pressed", self.set_up, clock_id, timezone_time)
        delete_button.connect("pressed", self.delete_clock, clock_id)
        clockname_label.connect("changed", self.set_clockname, clock_id)

        label_cont = clockname_label.get_style_context()
        label_cont.add_class("label")
        Gtk.StyleContext.add_provider(
            label_cont,
            self.provider,
            Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION,
        )
        self.maingrid.show_all()
        # update clockdata
        self.clocklist[clock_id] = {
            "hour_img": hour_image,
            "minute_img": minute_image,
            "misc_widgets": [
                bg, delete_button, trashbox, delete_button, offset_down,
                offset_up, timezone_time, timezone_box, clockname_label,
            ],
            "offset": offset,
            "clockname_label": clockname_label,
            "clocklabel": clockname,
            "ampm": ampm_label,
        }

        # save new data to file, refresh interface
        self.save_tofile()
        curr_time = time.time()
        self.refresh_clocks(curr_time)

    def waitfornext(self):
        # fetch current epoch and time until next minute (break time - 1 sec)
        startt = time.time()
        wait = 61 - (int(startt) % 60)
        return startt, wait

    def update_images(self, t, clockdata):
        # single clock update of images
        offset = clockdata["offset"]
        showtime = (t + offset)
        # am/pm?
        am = "a.m." if showtime % 86400 < 43200 else "p.m."
        # defining time to find image-indexes, images
        hour = int((showtime % 43200) / 720)
        minute = int((showtime % 3600) / 60)
        hr_index = 60 - hour if hour != 0 else 0
        min_index = 60 - minute if minute != 0 else 0
        # update images
        clockdata["hour_img"].set_from_pixbuf(hours[hr_index - 60])
        clockdata["minute_img"].set_from_pixbuf(minutes[min_index - 60])
        set_ampm = clockdata["ampm"]
        set_ampm.set_text(am)
        self.show_all()

    def refresh_clocks(self, currepoch=None):
        currepoch = currepoch if currepoch else time.time()
        # refresh all widgets
        for clockdata in [self.clocklist[k] for k in self.clocklist.keys()]:
            GObject.idle_add(
                self.update_images, currepoch, clockdata,
                priority=GObject.PRIORITY_DEFAULT,
            )

    def update_gmt(self):
        # The loop, sleep is set automaically (appr 1 minute after first cycle)
        while True:
            timedata = self.waitfornext()
            curr_time = timedata[0]
            self.refresh_clocks(curr_time)
            time.sleep(timedata[1])

    def set_down(self, button, clock, label):
        # set previous step in timeshift
        self.set_timeshift(button, clock, label, down=True)

    def set_up(self, button, clock, label):
        # set next step in timeshift
        self.set_timeshift(button, clock, label, down=False)

    def set_timeshift(self, button, clock, label, down):
        # sets the time shift on combo change
        curr_offset = self.clocklist[clock]["offset"]
        nxt = -1 if down else +1
        newindex = tz_data.index(
            self.convert_offset_tolabel(curr_offset)
        ) + nxt
        newindex = 0 if newindex < 0 else newindex
        try:
            newlabel = tz_data[newindex]
        except IndexError:
            newlabel = tz_data[-1]
        label.set_text(newlabel)
        self.clocklist[clock]["offset"] = self.convert_label_tooffset(newlabel)
        curr_time = time.time()
        self.refresh_clocks(curr_time)
        self.save_tofile()

    def set_clockname(self, entry, clock):
        # set the time shift on combo change
        newlabel = entry.get_text()
        self.clocklist[clock]["clocklabel"] = newlabel
        self.save_tofile()

    def delete_clock(self, button, clock):
        if len(self.clocklist) > 1:
            # delete the clock; data and all corresponding widgets
            misc_widgets = self.clocklist[clock]["misc_widgets"]
            hour_image = self.clocklist[clock]["hour_img"]
            minute_image = self.clocklist[clock]["minute_img"]
            ampm_mention = self.clocklist[clock]["ampm"]
            for w in misc_widgets + [hour_image, minute_image, ampm_mention]:
                w.destroy()
            del self.clocklist[clock]
            self.save_tofile()

    def on_press(self, box, arg):
        self.manager.show_popover(self.box)

    def do_update_popovers(self, manager):
        self.manager = manager
        self.manager.register_popover(self.box, self.popover)



#!/usr/bin/env python3

"""
Workspace Overview
Author: Jacob Vlijm
Copyright © 2017-2019 Ubuntu Budgie Developers
Website: https://ubuntubudgie.org
This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or any later version. This
program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details. You
should have received a copy of the GNU General Public License along with this
program.  If not, see <http://www.gnu.org/licenses/>.
"""

import subprocess
from itertools import groupby
from operator import itemgetter

# wm_classes to be ignored
ignore = [
    '"budgie-panel", "Budgie-panel"', '"desktop_window", "Nautilus"',
    '"plank", "Plank"', '"Showtime", "showtime"', None,
]


def get(cmd):
    # just a helper. (re-) try/except for buggy wmctrl
    try:
        return subprocess.check_output(cmd).decode("utf-8").strip()
    except subprocess.CalledProcessError:
        pass


def getspaces(wsdata):
    # get the current workspace and n- spaces from wmctrl -d
    newsrc = wsdata.splitlines()
    n_ws = len(newsrc)
    curr_ws = [l.split()[0] for l in newsrc if "*" in l][0]
    return n_ws, curr_ws


def show_wmclass(wid):
    # get WM_CLASS from window- id
    try:
        return get(["xprop", "-id", wid, "WM_CLASS"]).split("=")[-1].strip()
    except (IndexError, AttributeError):
        pass


def get_wmname(wid):
    # get WM_NAME from window- id
    try:
        return get([
            "xprop", "-id", wid, "WM_NAME"
        ]).split("=")[-1].strip().strip('"')
    except (IndexError, AttributeError):
        pass


def get_menuset(wdata):
    # return valid window-ids and workspaces from wmctrl -l
    allwindows = [l.split() for l in wdata.splitlines()]
    return [wid[1] for wid in allwindows if not show_wmclass(wid[0]) in ignore]


def new_icon(currws):
    # arrange icon path from current workspace
    return "ws" + str(int(currws) + 1) + "-symbolic"


def update_winmenu(currdata):
    newdata = get(["wmctrl", "-l"])
    newmenudata = []
    try:
        test = currdata[0] == newdata
    except TypeError:
        test = False
    if all([not test, newdata]):
        for l in [w.split() for w in newdata.splitlines()]:
            wid = l[0]
            wspace = l[1]
            wname = get_wmname(wid)
            wmclass = show_wmclass(wid)
            if all([wname, wmclass, wmclass not in ignore]):
                appname = wmclass.split(",")[-1].strip().strip('"')
                newmenudata.append([int(wspace), appname, wname, wid])
        newmenudata = sorted(newmenudata, key=itemgetter(0, 1, 2))
        newmenu = []
        subspace = []
        for wsp, data in groupby(newmenudata, itemgetter(0)):
            subspace.append(wsp)
            wlst = [w[1:] for w in list(data)]
            for app, windows in groupby(wlst, itemgetter(0)):
                subspace.append([app, [w[1:] for w in list(windows)]])
            newmenu.append(subspace)
            subspace = []
        return (newdata, newmenu)
    else:
        return currdata
